#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>

// The way to launch a new program is to fork(), then call one of
// the functions from the exec(3) family (see "man 3 exec").
// Those functions replace the current process by one that executes
// the program specified in the parameters. That process remains a
// child of its parent process and can be waited for.
//
// The exec functions are not meant to return; if they do, then there
// was an error trying to run the other program.

int main ()
{
	if (fork()) {

		// Parent process: wait till child terminates (ignoring
		// its result), then print "terminated" and exit.
		wait(NULL);
		printf("[parent] child terminated!\n");

	} else {

		// Child process

		printf("[child] waiting for you to type <return>\n");
		getchar();

		// The following two variants (execlp/execvp) are equivalent,
		// where the arguments start with the "zeroth" argument -
		// normally the name of the programme itself.

		execlp("ls","ls","-l", NULL);
		// char *args[] = { "ls", "ls", "-l", NULL };
		// execvp("ls",args);

		// The following command is reached only when exec() fails.
		printf("This should not happen\n");

	}
}
